from popolo.views import OrganizationDetailView, PersonDetailView, \
    MembershipDetailView, PostDetailView
from django.conf.urls import url

__author__ = 'guglielmo'

urlpatterns = [
    url(r'^person/(?P<slug>[-\w]+)/$', PersonDetailView.as_view(),
        name='person-detail'),
    url(r'^organization/(?P<slug>[-\w]+)/$', OrganizationDetailView.as_view(),
        name='organization-detail'),
    url(r'^membership/(?P<slug>[-\w]+)/$', MembershipDetailView.as_view(),
        name='membership-detail'),
    url(r'^post/(?P<slug>[-\w]+)/$', PostDetailView.as_view(),
        name='post-detail'),
]



from django.db.models import Q

__author__ = 'guglielmo'

from django.db import models
from datetime import datetime


class DateframeableQuerySet(models.query.QuerySet):
    """
    A custom ``QuerySet`` allowing easy retrieval of current, past and future
    instances
    of a Dateframeable model.

    Here, a *Dateframeable model* denotes a model class having an associated
    date range.

    We assume that the date range is described by two ``Char`` fields
    named ``start_date`` and ``end_date``, respectively,
    whose validation pattern is: "^[0-9]{4}(-[0-9]{2}){0,2}$",
    in order to represent partial dates.
    """

    def past(self, moment=None):
        """
        Return a QuerySet containing the *past* instances of the model
        (i.e. those having an end date which is in the past).
        """
        if moment is None:
            moment = datetime.strftime(datetime.now(), '%Y-%m-%d')
        return self.filter(end_date__lte=moment)

    def future(self, moment=None):
        """
        Return a QuerySet containing the *future* instances of the model
        (i.e. those having a start date which is in the future).
        """
        if moment is None:
            moment = datetime.strftime(datetime.now(), '%Y-%m-%d')
        return self.filter(start_date__gte=moment)

    def current(self, moment=None):
        """
        Return a QuerySet containing the *current* instances of the model
        at the given moment in time, if the parameter is spcified
        now if it is not
        @moment - is a string, representing a date in the YYYY-MM-DD format
        (i.e. those for which the moment date-time lies within their
        associated time range).
        """
        if moment is None:
            moment = datetime.strftime(datetime.now(), '%Y-%m-%d')

        return self.filter(Q(start_date__lte=moment) &
                           (Q(end_date__gte=moment) | Q(end_date__isnull=True)))


class PersonQuerySet(DateframeableQuerySet):
    pass


class OrganizationQuerySet(DateframeableQuerySet):
    pass


class PostQuerySet(DateframeableQuerySet):
    pass


class MembershipQuerySet(DateframeableQuerySet):
    pass


class ContactDetailQuerySet(DateframeableQuerySet):
    pass


class OtherNameQuerySet(DateframeableQuerySet):
    pass



from django.views.generic import DetailView
from popolo.models import Organization, Person, Membership, Post


class PersonDetailView(DetailView):
    model = Person
    context_object_name = 'person'
    template_name = 'person_detail.html'


class OrganizationDetailView(DetailView):
    model = Organization
    context_object_name = 'organization'
    template_name = 'organization_detail.html'


class MembershipDetailView(DetailView):
    model = Membership
    context_object_name = 'membership'
    template_name = 'membership_detail.html'


class PostDetailView(DetailView):
    model = Post
    context_object_name = 'post'
    template_name = 'post_detail.html'



try:
    from django.contrib.contenttypes.admin import GenericTabularInline, \
        GenericStackedInline
except ImportError:
    from django.contrib.contenttypes.generic import GenericTabularInline, \
        GenericStackedInline
from popolo import models


class LinkAdmin(GenericTabularInline):
    model = models.Link
    extra = 0


class IdentifierAdmin(GenericTabularInline):
    model = models.Identifier
    extra = 0


class ContactDetailAdmin(GenericStackedInline):
    model = models.ContactDetail
    extra = 0


class OtherNameAdmin(GenericTabularInline):
    model = models.OtherName
    extra = 0


class SourceAdmin(GenericTabularInline):
    model = models.Source
    extra = 0


BASE_INLINES = [
    LinkAdmin, IdentifierAdmin, ContactDetailAdmin, OtherNameAdmin, SourceAdmin
]



"""
Implements tests specific to the popolo module.
Run with "manage.py test popolo, or with python".
"""

from django.test import TestCase
from popolo.behaviors.tests import TimestampableTests, DateframeableTests
from popolo.models import Person, Organization, Post, ContactDetail
from faker import Factory

faker = Factory.create('it_IT')  # a factory to create fake names for tests


class PersonTestCase(DateframeableTests, TimestampableTests, TestCase):
    model = Person
    object_name = 'person'

    def create_instance(self, **kwargs):
        if 'name' not in kwargs:
            kwargs.update({'name': u'test instance'})
        return Person.objects.create(**kwargs)

    def test_add_membership(self):
        p = self.create_instance(name=faker.name(), birth_date=faker.year())
        o = Organization.objects.create(name=faker.company())
        p.add_membership(o)
        self.assertEqual(p.memberships.count(), 1)

    def test_add_memberships(self):
        p = self.create_instance(name=faker.name(), birth_date=faker.year())
        os = [
            Organization.objects.create(name=faker.company())
            for i in range(3)
        ]
        p.add_memberships(os)
        self.assertEqual(p.memberships.count(), 3)

    def test_add_role(self):
        p = self.create_instance(name=faker.name(), birth_date=faker.year())
        o = Organization.objects.create(name=faker.company())
        r = Post.objects.create(label=u'CEO', organization=o)
        p.add_role(r)
        self.assertEqual(p.memberships.count(), 1)

    def test_add_contact_detail(self):
        p = self.create_instance()
        p.add_contact_detail(contact_type=ContactDetail.CONTACT_TYPES.email,
                             value=faker.email())
        self.assertEqual(p.contact_details.count(), 1)

    def test_add_contact_details(self):
        p = self.create_instance()
        contacts = [
            {'contact_type': ContactDetail.CONTACT_TYPES.email,
             'value': faker.email()},
            {'contact_type': ContactDetail.CONTACT_TYPES.phone,
             'value': faker.phone_number()},

        ]
        p.add_contact_details(contacts)
        self.assertEqual(p.contact_details.count(), 2)

    def test_it_copies_birth_date_after_saving(self):
        pr = Person(name=faker.name(), birth_date=faker.year())
        self.assertIsNone(pr.start_date)
        pr.save()
        self.assertEqual(pr.start_date, pr.birth_date)

    def test_it_copies_death_date_after_saving(self):
        pr = Person(name=faker.name(), death_date=faker.year())
        self.assertIsNone(pr.end_date)
        pr.save()
        self.assertEqual(pr.end_date, pr.death_date)

    def test_add_links_and_sources(self):
        p = self.create_instance()
        p.links.create(url='http://link.example.org/', note='Note')
        p.sources.create(url='http://source.example.org/', note='Source note')
        self.assertEqual(p.links.count(), 1)
        self.assertEqual(
            p.sources.filter(url='http://link.example.org/').count(), 0)


class OrganizationTestCase(DateframeableTests, TimestampableTests, TestCase):
    model = Organization
    object_name = 'organization'

    def create_instance(self, **kwargs):
        if 'name' not in kwargs:
            kwargs.update({'name': u'test instance'})
        return Organization.objects.create(**kwargs)

    def test_add_member(self):
        o = self.create_instance(name=faker.company())
        p = Person.objects.create(name=faker.name(), birth_date=faker.year())
        o.add_member(p)
        self.assertEqual(o.memberships.count(), 1)

    def test_add_members(self):
        o = self.create_instance(name=faker.company())
        ps = [
            Person.objects.create(name=faker.name(), birth_date=faker.year()),
            Person.objects.create(name=faker.name(), birth_date=faker.year()),
            Person.objects.create(name=faker.name(), birth_date=faker.year()),
        ]
        o.add_members(ps)
        self.assertEqual(o.memberships.count(), 3)

    def test_add_post(self):
        o = Organization.objects.create(name=faker.company())
        o.add_post(label=u'CEO')
        self.assertEqual(o.posts.count(), 1)

    def test_add_posts(self):
        o = Organization.objects.create(name=faker.company())
        o.add_posts([
            {'label': u'Presidente'},
            {'label': u'Vicepresidente'},
        ])
        self.assertEqual(o.posts.count(), 2)

    def test_it_copies_the_foundation_date_to_start_date(self):
        o = Organization(name=faker.company(), founding_date=faker.year())
        # it is not set to start_date until saved
        self.assertIsNone(o.start_date)
        o.save()
        self.assertEqual(o.start_date, o.founding_date)

    def test_it_copies_the_dissolution_date_to_end_date(self):
        o = Organization(name=faker.company(), dissolution_date=faker.year())
        # it is not set to start_date until saved
        self.assertIsNone(o.end_date)
        o.save()
        self.assertEqual(o.end_date, o.dissolution_date)


class PostTestCase(DateframeableTests, TimestampableTests, TestCase):
    model = Post

    def create_instance(self, **kwargs):
        if 'label' not in kwargs:
            kwargs.update({'label': u'test instance'})
        if 'other_label' not in kwargs:
            kwargs.update({'other_label': u'TI,TEST'})

        if 'organization' not in kwargs:
            o = Organization.objects.create(name=faker.company())
            kwargs.update({'organization': o})
        return Post.objects.create(**kwargs)

    def test_add_person(self):
        o = Organization.objects.create(name=faker.company())
        p = self.create_instance(label=u'Chief Executive Officer',
                                 other_label=u'CEO,AD', organization=o)
        pr = Person.objects.create(name=faker.name(), birth_date=faker.year())
        p.add_person(pr)
        self.assertEqual(p.memberships.count(), 1)



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

try:
    from django.contrib.contenttypes.admin import GenericTabularInline
except ImportError:
    from django.contrib.contenttypes.generic import GenericTabularInline

from popolo import models
from .behaviors import admin as generics
from django.utils.translation import ugettext_lazy as _


class MembershipInline(admin.StackedInline):
    extra = 0
    model = models.Membership


class PersonAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('name', 'gender', 'birth_date', 'death_date')
        }),
        (_('Biography'), {
            'classes': ('collapse',),
            'fields': ('summary', 'image', 'biography')
        }),
        (_('Honorifics'), {
            'classes': ('collapse',),
            'fields': ('honorific_prefix', 'honorific_suffix')
        }),
        (_('Demography'), {
            'classes': ('collapse',),
            'fields': ('national_identity',)
        }),
        (_('Special Names'), {
            'classes': ('collapse',),
            'fields': ('family_name', 'given_name',
                       'additional_name', 'patronymic_name', 'sort_name')
        }),
        (_('Advanced options'), {
            'classes': ('collapse',),
            'fields': ('start_date', 'end_date')
        }),
    )
    inlines = generics.BASE_INLINES + [MembershipInline]


class OrganizationMembersInline(MembershipInline):
    verbose_name = _("Member")
    verbose_name_plural = _("Members of this organization")
    fk_name = 'organization'


class OrganizationOnBehalfInline(MembershipInline):
    verbose_name = _("Proxy member")
    verbose_name_plural = _("Members acting on behalf of this organization")
    fk_name = 'on_behalf_of'


class PostAdmin(admin.ModelAdmin):
    model = models.Post
    fieldsets = (
        (None, {
            'fields': ('label', 'role', 'start_date', 'end_date')
        }),
        (_('Details'), {
            'classes': ('collapse',),
            'fields': ('other_label', 'area', 'organization')
        }),
    )
    inlines = [
        generics.LinkAdmin,
        generics.ContactDetailAdmin,
        generics.SourceAdmin
    ]


class OrganizationAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('name', 'founding_date', 'dissolution_date')
        }),
        (_('Details'), {
            'classes': ('collapse',),
            'fields': ('summary', 'image', 'description')
        }),
        (_('Advanced options'), {
            'classes': ('collapse',),
            'fields': (
                'classification',
                'parent', 'area',
                'start_date', 'end_date'
            )
        }),
    )
    inlines = generics.BASE_INLINES + [
        OrganizationMembersInline,
        OrganizationOnBehalfInline
    ]

class AreaI18NameInlineAdmin(admin.StackedInline):
    extra = 0
    model = models.AreaI18Name

class AreaAdmin(admin.ModelAdmin):
    model = models.Area
    fieldsets = (
        (None, {
            'fields': (
                'name', 'identifier', 'classification', 'parent'
            )
        }),
        (_('Details'), {
            'classes': ('collapse',),
            'fields': (
                'classification',
                'inhabitants',
            )
        }),
    )
    inlines = [
        AreaI18NameInlineAdmin,
        generics.SourceAdmin

    ]

admin.site.register(models.Post, PostAdmin)
admin.site.register(models.Person, PersonAdmin)
admin.site.register(models.Organization, OrganizationAdmin)
admin.site.register(models.Area, AreaAdmin)
admin.site.register(models.Language)


# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-09-15 22:39
from __future__ import unicode_literals

import autoslug.fields
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.text
import django.utils.timezone
import model_utils.fields
import popolo.behaviors.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Area',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField(null=True)),
                ('start_date', models.CharField(blank=True, help_text='The date when the validity of the item starts', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='start date')),
                ('end_date', models.CharField(blank=True, help_text='The date when the validity of the item ends', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='end date')),
                ('created_at', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='creation time')),
                ('updated_at', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='last modification time')),
                ('slug', autoslug.fields.AutoSlugField(editable=False, max_length=255, populate_from=popolo.behaviors.models.get_slug_source, slugify=django.utils.text.slugify, unique=True)),
                ('name', models.CharField(blank=True, help_text='A primary name', max_length=256, verbose_name='name')),
                ('identifier', models.CharField(blank=True, help_text='An issued identifier', max_length=128, verbose_name='identifier')),
                ('classification', models.CharField(blank=True, help_text='An area category, e.g. city', max_length=128, verbose_name='classification')),
                ('geom', models.TextField(blank=True, help_text='A geometry', null=True, verbose_name='geom')),
                ('inhabitants', models.IntegerField(blank=True, help_text='The total number of inhabitants', null=True, verbose_name='inhabitants')),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('parent', models.ForeignKey(blank=True, help_text='The area that contains this area', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='popolo.Area', verbose_name='Parent')),
            ],
            options={
                'verbose_name': 'Geographic Area',
                'verbose_name_plural': 'Geographic Areas',
            },
        ),
        migrations.CreateModel(
            name='AreaI18Name',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='name')),
                ('area', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='i18n_names', to='popolo.Area')),
            ],
            options={
                'verbose_name': 'I18N Name',
                'verbose_name_plural': 'I18N Names',
            },
        ),
        migrations.CreateModel(
            name='ContactDetail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField(null=True)),
                ('start_date', models.CharField(blank=True, help_text='The date when the validity of the item starts', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='start date')),
                ('end_date', models.CharField(blank=True, help_text='The date when the validity of the item ends', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='end date')),
                ('created_at', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='creation time')),
                ('updated_at', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='last modification time')),
                ('label', models.CharField(blank=True, help_text='A human-readable label for the contact detail', max_length=256, verbose_name='label')),
                ('contact_type', models.CharField(choices=[('ADDRESS', 'Address'), ('EMAIL', 'Email'), ('URL', 'Url'), ('MAIL', 'Snail mail'), ('TWITTER', 'Twitter'), ('FACEBOOK', 'Facebook'), ('PHONE', 'Telephone'), ('MOBILE', 'Mobile'), ('TEXT', 'Text'), ('VOICE', 'Voice'), ('FAX', 'Fax'), ('CELL', 'Cell'), ('VIDEO', 'Video'), ('PAGER', 'Pager'), ('TEXTPHONE', 'Textphone')], help_text="A type of medium, e.g. 'fax' or 'email'", max_length=12, verbose_name='type')),
                ('value', models.CharField(help_text='A value, e.g. a phone number or email address', max_length=256, verbose_name='value')),
                ('note', models.CharField(blank=True, help_text='A note, e.g. for grouping contact details by physical location', max_length=512, verbose_name='note')),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'Contact detail',
                'verbose_name_plural': 'Contact details',
            },
        ),
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='creation time')),
                ('updated_at', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='last modification time')),
                ('name', models.CharField(help_text="The event's name", max_length=128, verbose_name='name')),
                ('description', models.CharField(blank=True, help_text="The event's description", max_length=512, null=True, verbose_name='description')),
                ('start_date', models.CharField(blank=True, help_text='The time at which the event starts', max_length=20, null=True, validators=[django.core.validators.RegexValidator(code='invalid_start_date', message='start date must follow the given pattern: ^[0-9]{4}((-[0-9]{2}){0,2}|(-[0-9]{2}){2}T[0-9]{2}(:[0-9]{2}){0,2}(Z|[+-][0-9]{2}(:[0-9]{2})?))$', regex='^[0-9]{4}((-[0-9]{2}){0,2}|(-[0-9]{2}){2}T[0-9]{2}(:[0-9]{2}){0,2}(Z|[+-][0-9]{2}(:[0-9]{2})?))$')], verbose_name='start date')),
                ('end_date', models.CharField(blank=True, help_text='The time at which the event ends', max_length=20, null=True, validators=[django.core.validators.RegexValidator(code='invalid_end_date', message='end date must follow the given pattern: ^[0-9]{4}((-[0-9]{2}){0,2}|(-[0-9]{2}){2}T[0-9]{2}(:[0-9]{2}){0,2}(Z|[+-][0-9]{2}(:[0-9]{2})?))$', regex='^[0-9]{4}((-[0-9]{2}){0,2}|(-[0-9]{2}){2}T[0-9]{2}(:[0-9]{2}){0,2}(Z|[+-][0-9]{2}(:[0-9]{2})?))$')], verbose_name='end date')),
                ('location', models.CharField(blank=True, help_text="The event's location", max_length=255, null=True, verbose_name='location')),
                ('status', models.CharField(blank=True, help_text="The event's status", max_length=128, null=True, verbose_name='status')),
                ('classification', models.CharField(blank=True, help_text="The event's category", max_length=128, null=True, verbose_name='classification')),
                ('area', models.ForeignKey(blank=True, help_text='The Area the Event is related to', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='events', to='popolo.Area')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Identifier',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField(null=True)),
                ('start_date', models.CharField(blank=True, help_text='The date when the validity of the item starts', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='start date')),
                ('end_date', models.CharField(blank=True, help_text='The date when the validity of the item ends', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='end date')),
                ('identifier', models.CharField(help_text='An issued identifier, e.g. a DUNS number', max_length=512, verbose_name='identifier')),
                ('scheme', models.CharField(blank=True, help_text='An identifier scheme, e.g. DUNS', max_length=128, verbose_name='scheme')),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'Identifier',
                'verbose_name_plural': 'Identifiers',
            },
        ),
        migrations.CreateModel(
            name='Language',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('dbpedia_resource', models.CharField(help_text='DbPedia URI of the resource', max_length=255, unique=True, verbose_name='dbpedia resource')),
                ('iso639_1_code', models.CharField(help_text='ISO 639_1 code, ex: en, it, de, fr, es, ...', max_length=2, verbose_name='iso639_1 code')),
                ('name', models.CharField(help_text='English name of the language', max_length=128, verbose_name='name')),
            ],
            options={
                'verbose_name': 'Language',
                'verbose_name_plural': 'Languages',
            },
        ),
        migrations.CreateModel(
            name='Link',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField(null=True)),
                ('url', models.URLField(help_text='A URL', max_length=350, verbose_name='url')),
                ('note', models.CharField(blank=True, help_text="A note, e.g. 'Wikipedia page'", max_length=512, verbose_name='note')),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'Link',
                'verbose_name_plural': 'Links',
            },
        ),
        migrations.CreateModel(
            name='Membership',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start_date', models.CharField(blank=True, help_text='The date when the validity of the item starts', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='start date')),
                ('end_date', models.CharField(blank=True, help_text='The date when the validity of the item ends', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='end date')),
                ('created_at', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='creation time')),
                ('updated_at', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='last modification time')),
                ('slug', autoslug.fields.AutoSlugField(editable=False, max_length=255, populate_from=popolo.behaviors.models.get_slug_source, slugify=django.utils.text.slugify, unique=True)),
                ('label', models.CharField(blank=True, help_text='A label describing the membership', max_length=256, null=True, verbose_name='label')),
                ('role', models.CharField(blank=True, help_text='The role that the member fulfills in the organization', max_length=256, null=True, verbose_name='role')),
                ('area', models.ForeignKey(blank=True, help_text='The geographic area to which the post is related', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='memberships', to='popolo.Area', verbose_name='Area')),
            ],
            options={
                'verbose_name': 'Membership',
                'verbose_name_plural': 'Memberships',
            },
        ),
        migrations.CreateModel(
            name='Organization',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start_date', models.CharField(blank=True, help_text='The date when the validity of the item starts', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='start date')),
                ('end_date', models.CharField(blank=True, help_text='The date when the validity of the item ends', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='end date')),
                ('created_at', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='creation time')),
                ('updated_at', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='last modification time')),
                ('slug', autoslug.fields.AutoSlugField(editable=False, max_length=255, populate_from=popolo.behaviors.models.get_slug_source, slugify=django.utils.text.slugify, unique=True)),
                ('name', models.CharField(help_text='A primary name, e.g. a legally recognized name', max_length=128, verbose_name='name')),
                ('classification', models.CharField(blank=True, help_text='An organization category, e.g. committee', max_length=64, null=True, verbose_name='classification')),
                ('abstract', models.CharField(blank=True, help_text='A one-line description of an organization', max_length=256, null=True, verbose_name='abstract')),
                ('description', models.TextField(blank=True, help_text='An extended description of an organization', null=True, verbose_name='biography')),
                ('founding_date', models.CharField(blank=True, help_text='A date of founding', max_length=10, null=True, validators=[django.core.validators.RegexValidator(code='invalid_founding_date', message='founding date must follow the given pattern: ^[0-9]{4}(-[0-9]{2}){0,2}$', regex='^[0-9]{4}(-[0-9]{2}){0,2}$')], verbose_name='founding date')),
                ('dissolution_date', models.CharField(blank=True, help_text='A date of dissolution', max_length=10, null=True, validators=[django.core.validators.RegexValidator(code='invalid_dissolution_date', message='dissolution date must follow the given pattern: ^[0-9]{4}(-[0-9]{2}){0,2}$', regex='^[0-9]{4}(-[0-9]{2}){0,2}$')], verbose_name='dissolution date')),
                ('image', models.URLField(blank=True, help_text='A URL of an image, to identify the organization visually', max_length=255, null=True, verbose_name='image')),
                ('area', models.ForeignKey(blank=True, help_text='The geographic area to which this organization is related', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='organizations', to='popolo.Area')),
                ('parent', models.ForeignKey(blank=True, help_text='The organization that contains this organization', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='popolo.Organization', verbose_name='Parent')),
            ],
            options={
                'verbose_name': 'Organization',
                'verbose_name_plural': 'Organizations',
            },
        ),
        migrations.CreateModel(
            name='OtherName',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField(null=True)),
                ('start_date', models.CharField(blank=True, help_text='The date when the validity of the item starts', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='start date')),
                ('end_date', models.CharField(blank=True, help_text='The date when the validity of the item ends', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='end date')),
                ('name', models.CharField(help_text='An alternate or former name', max_length=512, verbose_name='name')),
                ('note', models.CharField(blank=True, help_text="A note, e.g. 'Birth name'", max_length=1024, null=True, verbose_name='note')),
                ('source', models.URLField(blank=True, help_text='The URL of the source where this information comes from', max_length=256, null=True, verbose_name='source')),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'Other name',
                'verbose_name_plural': 'Other names',
            },
        ),
        migrations.CreateModel(
            name='Person',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start_date', models.CharField(blank=True, help_text='The date when the validity of the item starts', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='start date')),
                ('end_date', models.CharField(blank=True, help_text='The date when the validity of the item ends', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='end date')),
                ('created_at', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='creation time')),
                ('updated_at', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='last modification time')),
                ('slug', autoslug.fields.AutoSlugField(editable=False, max_length=255, populate_from=popolo.behaviors.models.get_slug_source, slugify=django.utils.text.slugify, unique=True)),
                ('name', models.CharField(help_text="A person's preferred full name", max_length=512, verbose_name='name')),
                ('family_name', models.CharField(blank=True, help_text='One or more family names', max_length=128, null=True, verbose_name='family name')),
                ('given_name', models.CharField(blank=True, help_text='One or more primary given names', max_length=128, null=True, verbose_name='given name')),
                ('additional_name', models.CharField(blank=True, help_text='One or more secondary given names', max_length=128, null=True, verbose_name='additional name')),
                ('honorific_prefix', models.CharField(blank=True, help_text="One or more honorifics preceding a person's name", max_length=32, null=True, verbose_name='honorific prefix')),
                ('honorific_suffix', models.CharField(blank=True, help_text="One or more honorifics following a person's name", max_length=32, null=True, verbose_name='honorific suffix')),
                ('patronymic_name', models.CharField(blank=True, help_text='One or more patronymic names', max_length=128, null=True, verbose_name='patronymic name')),
                ('sort_name', models.CharField(blank=True, help_text='A name to use in an lexicographically ordered list', max_length=128, null=True, verbose_name='sort name')),
                ('email', models.EmailField(blank=True, help_text='A preferred email address', max_length=254, null=True, verbose_name='email')),
                ('gender', models.CharField(blank=True, help_text='A gender', max_length=32, verbose_name='gender')),
                ('birth_date', models.CharField(blank=True, help_text='A date of birth', max_length=10, null=True, verbose_name='birth date')),
                ('death_date', models.CharField(blank=True, help_text='A date of death', max_length=10, null=True, verbose_name='death date')),
                ('image', models.URLField(blank=True, help_text='A URL of a head shot', null=True, verbose_name='image')),
                ('summary', models.CharField(blank=True, help_text="A one-line account of a person's life", max_length=1024, null=True, verbose_name='summary')),
                ('biography', models.TextField(blank=True, help_text="An extended account of a person's life", null=True, verbose_name='biography')),
                ('national_identity', models.CharField(blank=True, help_text='A national identity', max_length=128, null=True, verbose_name='national identity')),
            ],
            options={
                'verbose_name': 'Person',
                'verbose_name_plural': 'People',
            },
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start_date', models.CharField(blank=True, help_text='The date when the validity of the item starts', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='start date')),
                ('end_date', models.CharField(blank=True, help_text='The date when the validity of the item ends', max_length=10, null=True, validators=[django.core.validators.RegexValidator(message='Date has wrong format', regex='^[0-9]{4}(-[0-9]{2}){0,2}$'), popolo.behaviors.models.validate_partial_date], verbose_name='end date')),
                ('created_at', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='creation time')),
                ('updated_at', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='last modification time')),
                ('slug', autoslug.fields.AutoSlugField(editable=False, max_length=255, populate_from=popolo.behaviors.models.get_slug_source, slugify=django.utils.text.slugify, unique=True)),
                ('label', models.CharField(blank=True, help_text='A label describing the post', max_length=256, verbose_name='label')),
                ('other_label', models.CharField(blank=True, help_text='An alternate label, such as an abbreviation', max_length=32, null=True, verbose_name='other label')),
                ('role', models.CharField(blank=True, help_text='The function that the holder of the post fulfills', max_length=256, null=True, verbose_name='role')),
                ('area', models.ForeignKey(blank=True, help_text='The geographic area to which the post is related', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='posts', to='popolo.Area', verbose_name='Area')),
                ('organization', models.ForeignKey(help_text='The organization in which the post is held', on_delete=django.db.models.deletion.CASCADE, related_name='posts', to='popolo.Organization', verbose_name='Organization')),
            ],
            options={
                'verbose_name': 'Post',
                'verbose_name_plural': 'Posts',
            },
        ),
        migrations.CreateModel(
            name='Source',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField(null=True)),
                ('url', models.URLField(help_text='A URL', max_length=350, verbose_name='url')),
                ('note', models.CharField(blank=True, help_text="A note, e.g. 'Parliament website'", max_length=512, verbose_name='note')),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'Source',
                'verbose_name_plural': 'Sources',
            },
        ),
        migrations.AddField(
            model_name='membership',
            name='member_organization',
            field=models.ForeignKey(blank=True, help_text='The person who is a member of the organization', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='memberships_as_member', to='popolo.Organization', verbose_name='Person'),
        ),
        migrations.AddField(
            model_name='membership',
            name='on_behalf_of',
            field=models.ForeignKey(blank=True, help_text='The organization on whose behalf the person is a member of the organization', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='memberships_on_behalf_of', to='popolo.Organization', verbose_name='On behalf of'),
        ),
        migrations.AddField(
            model_name='membership',
            name='organization',
            field=models.ForeignKey(blank=True, help_text='The organization in which the person or organization is a member', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='memberships', to='popolo.Organization', verbose_name='Organization'),
        ),
        migrations.AddField(
            model_name='membership',
            name='person',
            field=models.ForeignKey(blank=True, help_text='The person who is a member of the organization', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='memberships', to='popolo.Person', verbose_name='Person'),
        ),
        migrations.AddField(
            model_name='membership',
            name='post',
            field=models.ForeignKey(blank=True, help_text='The post held by the person in the organization through this membership', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='memberships', to='popolo.Post', verbose_name='Post'),
        ),
        migrations.AddField(
            model_name='event',
            name='attendees',
            field=models.ManyToManyField(blank=True, help_text='People attending the event', related_name='attended_events', to='popolo.Person'),
        ),
        migrations.AddField(
            model_name='event',
            name='organization',
            field=models.ForeignKey(blank=True, help_text='The organization organizing the event', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='events', to='popolo.Organization'),
        ),
        migrations.AddField(
            model_name='event',
            name='parent',
            field=models.ForeignKey(blank=True, help_text='The Event that this event is part of', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='popolo.Event', verbose_name='Parent'),
        ),
        migrations.AddField(
            model_name='areai18name',
            name='language',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='popolo.Language', verbose_name='Language'),
        ),
        migrations.AlterUniqueTogether(
            name='areai18name',
            unique_together=set([('area', 'language', 'name')]),
        ),
    ]



# -*- coding: utf-8 -*-

from contextlib import contextmanager
import json
import os
import sys
from tempfile import NamedTemporaryFile

from mock import Mock, call
from django.utils import six

from django.core.exceptions import ValidationError
from django.core.management import call_command
from django.test import TestCase

from popolo.importers.popit import PopItImporter
from popolo import models


# There are some very basic tests here.  For when there is time, the
# following things could be added or improved:
#
#  * Test creation of sources
#  * Test creation of other names
#  * Test updates as well as creation of all objects
#  * Tests for the legislative_period_id in memberships special case
#  * Test setting of a parent area
#  * Test setting of a parent organisation
#  * Test handling of areas on posts
#  * Test the show_data_on_error context manager
#  * Test that objects that have disappeared are removed (not yet implemented)


@contextmanager
def capture_output():
    # Suggested here: http://stackoverflow.com/a/17981937/223092
    new_out, new_err = six.StringIO(), six.StringIO()
    old_out, old_err = sys.stdout, sys.stderr
    try:
        sys.stdout, sys.stderr = new_out, new_err
        yield new_out, new_err
    finally:
        sys.stdout, sys.stderr = old_out, old_err


class BasicImporterTests(TestCase):
    def test_all_top_level_optional(self):
        # This just check that you don't get an exception when
        # importing empty Popolo JSON.
        input_json = '{}'
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)

    def test_simplest_person(self):
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice"
        }

    ],
    "organizations": [],
    "memberships": []
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Person.objects.count(), 1)
        person = models.Person.objects.get()
        self.assertEqual(person.name, "Alice")
        self.assertEqual(
            person.identifiers.get(scheme='popit-person').identifier,
            "a1b2")

    def test_person_with_membership(self):
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice"
        }

    ],
    "organizations": [
        {
            "id": "commons",
            "name": "House of Commons"
        }
    ],
    "memberships": [
        {
            "person_id": "a1b2",
            "organization_id": "commons"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Membership.objects.count(), 1)
        self.assertEqual(models.Person.objects.count(), 1)
        self.assertEqual(models.Organization.objects.count(), 1)
        person = models.Person.objects.get()
        self.assertEqual(person.name, "Alice")
        organization = models.Organization.objects.get()
        self.assertEqual(organization.name, "House of Commons")
        self.assertEqual(
            organization.identifiers.get(
                scheme='popit-organization').identifier,
            "commons"
        )
        membership = models.Membership.objects.get()
        self.assertEqual(membership.person, person)
        self.assertEqual(membership.organization, organization)

    def test_import_from_a_file(self):
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice"
        }

    ],
    "organizations": [
        {
            "id": "commons",
            "name": "House of Commons"
        }
    ],
    "memberships": [
        {
            "person_id": "a1b2",
            "organization_id": "commons"
        }
    ]
}
'''
        try:
            with NamedTemporaryFile('w', delete=False, suffix='.json') as ntf:
                ntf.write(input_json)
            importer = PopItImporter()
            importer.import_from_export_json(ntf.name)
        finally:
            os.remove(ntf.name)
        # Now check everything was created:
        self.assertEqual(models.Membership.objects.count(), 1)
        self.assertEqual(models.Person.objects.count(), 1)
        self.assertEqual(models.Organization.objects.count(), 1)
        self.assertEqual(models.Identifier.objects.count(), 3)

    def test_person_with_inline_membership(self):
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice",
            "memberships": [
                {
                    "person_id": "a1b2",
                    "organization_id": "commons"
                }
            ]
        }

    ],
    "organizations": [
        {
            "id": "commons",
            "name": "House of Commons"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Membership.objects.count(), 1)
        self.assertEqual(models.Person.objects.count(), 1)
        self.assertEqual(models.Organization.objects.count(), 1)
        person = models.Person.objects.get()
        self.assertEqual(person.name, "Alice")
        organization = models.Organization.objects.get()
        self.assertEqual(organization.name, "House of Commons")
        self.assertEqual(
            organization.identifiers.get(
                scheme='popit-organization').identifier,
            "commons"
        )
        membership = models.Membership.objects.get()
        self.assertEqual(membership.person, person)
        self.assertEqual(membership.organization, organization)

    def test_custom_identifier_prefix(self):
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice"
        }

    ],
    "organizations": [
        {
            "id": "commons",
            "name": "House of Commons"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter(id_prefix='popolo:')
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Person.objects.count(), 1)
        self.assertEqual(models.Organization.objects.count(), 1)
        person = models.Person.objects.get()
        organization = models.Organization.objects.get()
        person_identifier = person.identifiers.get()
        organization_identifier = organization.identifiers.get()
        self.assertEqual(person_identifier.scheme, 'popolo:person')
        self.assertEqual(person_identifier.identifier, 'a1b2')
        self.assertEqual(organization_identifier.scheme, 'popolo:organization')
        self.assertEqual(organization_identifier.identifier, 'commons')

    def test_creates_new_person_if_not_found(self):
        existing_person = models.Person.objects.create(name='Algernon')
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Person.objects.count(), 2)
        new_person = models.Person.objects.exclude(pk=existing_person.id).get()
        new_person_identifier = new_person.identifiers.get()
        self.assertEqual(new_person_identifier.scheme, 'popit-person')
        self.assertEqual(new_person_identifier.identifier, 'a1b2')

    def test_updates_person_if_found(self):
        existing_person = models.Person.objects.create(name='Algernon')
        existing_person.identifiers.create(
            scheme='popolo:person',
            identifier="a1b2"
        )
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter(id_prefix='popolo:')
        mock_observer = Mock()
        importer.add_observer(mock_observer)
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Person.objects.count(), 1)
        # Reget the person from the database:
        person = models.Person.objects.get(pk=existing_person.id)
        self.assertEqual(person.name, 'Alice')
        # Check that the observer was called with created=False:
        self.assertEqual(mock_observer.notify.call_count, 1)
        self.assertEqual(
            mock_observer.notify.call_args,
            call(
                'person',
                existing_person,
                False,
                {
                    "id": "a1b2",
                    "name": "Alice"
                }
            )
        )

    def test_observer_called(self):
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice"
        }
    ],
    "organizations": [
        {
            "id": "commons",
            "name": "House of Commons"
        }
    ],
    "memberships": [
        {
            "person_id": "a1b2",
            "organization_id": "commons",
            "post_id": "65808"
        }
    ],
    "areas": [
        {
            "id": 100,
            "name": "Dulwich and West Norwood",
            "identifier": "gss:E14000673"
        }
    ],
    "posts": [
        {
            "id": "65808",
            "url": "https://candidates.democracyclub.org.uk/api/v0.9/posts/65808/",
            "label": "Member of Parliament for Dulwich and West Norwood",
            "role": "Member of Parliament",
            "organization_id": "commons",
            "group": "England",
            "area_id": 100
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        mock_observer = Mock()
        importer.add_observer(mock_observer)
        importer.import_from_export_json_data(data)
        # Just as a double-check, make sure that one of each of these
        # objects has been created:
        self.assertEqual(models.Person.objects.count(), 1)
        self.assertEqual(models.Organization.objects.count(), 1)
        self.assertEqual(models.Membership.objects.count(), 1)
        self.assertEqual(models.Area.objects.count(), 1)
        self.assertEqual(models.Post.objects.count(), 1)
        # Now check that the observer has been called for each one.
        self.assertEqual(mock_observer.notify.call_count, 5)
        # And check that the payload for each call is correct:
        self.assertEqual(
            mock_observer.notify.call_args_list,
            [
                call(
                    'area',
                    models.Area.objects.get(),
                    True,
                    {
                        "id": 100,
                        "name": "Dulwich and West Norwood",
                        "identifier": "gss:E14000673"
                    },
                ),
                call(
                    'organization',
                    models.Organization.objects.get(),
                    True,
                    {
                        "id": "commons",
                        "name": "House of Commons"
                    },
                ),
                call(
                    'post',
                    models.Post.objects.get(),
                    True,
                    {
                        "id": "65808",
                        "url":
                        "https://candidates.democracyclub.org.uk/api/v0.9"
                        "/posts/65808/",
                        "label": "Member of Parliament for Dulwich and West "
                        "Norwood",
                        "role": "Member of Parliament",
                        "organization_id": "commons",
                        "group": "England",
                        "area_id": 100
                    },
                ),
                call(
                    'person',
                    models.Person.objects.get(),
                    True,
                    {
                        'id': 'a1b2',
                        'name': 'Alice'
                    }
                ),
                call(
                    'membership',
                    models.Membership.objects.get(),
                    True,
                    {
                        'person_id': 'a1b2',
                        'organization_id': 'commons',
                        'id': 'missing_commons_missing_missing_missing_a1b2',
                        'post_id': '65808'
                    }
                )
            ]
        )

    def test_related_objects_for_person(self):
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice",
            "identifiers": [
                {
                    "identifier": "123456789",
                    "scheme": "yournextmp-candidate"
                }
            ],
            "contact_details": [
                {
                    "contact_type": "twitter",
                    "label": "",
                    "note": "",
                    "value": "sometwitterusernameorother"
                }

            ],
            "links": [
                {
                    "note": "homepage",
                    "url": "http://example.com/alice"
                }
            ]
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Person.objects.count(), 1)
        person = models.Person.objects.get()
        self.assertEqual(person.name, "Alice")
        self.assertEqual(person.identifiers.count(), 2)
        self.assertEqual(
            person.identifiers.get(scheme='popit-person').identifier,
            "a1b2")
        self.assertEqual(
            person.identifiers.get(scheme='yournextmp-candidate').identifier,
            "123456789")
        self.assertEqual(person.contact_details.count(), 1)
        contact_detail = person.contact_details.get()
        self.assertEqual(contact_detail.contact_type, 'twitter')
        self.assertEqual(contact_detail.value, 'sometwitterusernameorother')
        self.assertEqual(person.links.count(), 1)
        link = person.links.get()
        self.assertEqual(link.note, 'homepage')
        self.assertEqual(link.url, 'http://example.com/alice')

    def test_truncation_unknown_option(self):
        with self.assertRaises(ValueError):
            PopItImporter(truncate='dunno')

    def test_truncation_none(self):
        long_name = ('Albert ' * 100).strip()
        input_json = '''
{{
    "persons": [
        {{
            "id": "a1b2",
            "name": "{0}"
        }}
    ]
}}
'''.format(long_name)
        data = json.loads(input_json)
        # With truncate='yes', there should be no exception:
        importer = PopItImporter(truncate='yes')
        importer.import_from_export_json_data(data)
        person = models.Person.objects.get()
        max_length = person._meta.get_field('name').max_length
        truncated_name = long_name[:max_length]
        self.assertEqual(person.name, truncated_name)

    def test_truncation_exception(self):
        long_name = ('Albert ' * 100).strip()
        input_json = '''
{{
    "persons": [
        {{
            "id": "a1b2",
            "name": "{0}"
        }}
    ]
}}
'''.format(long_name)
        data = json.loads(input_json)
        # With the default, truncate='exception', there should be an
        # exception raised:
        importer = PopItImporter()
        with self.assertRaisesRegexp(
                ValidationError,
                '512'):
            # Capture the output just to reduce noise in the test
            # output - this would include output from
            # show_data_on_error otherwise.
            with capture_output() as (out, err):
                importer.import_from_export_json_data(data)

    def test_truncation_warn(self):
        long_name = ('Albert ' * 100).strip()
        input_json = '''
{{
    "persons": [
        {{
            "id": "a1b2",
            "name": "{0}"
        }}
    ]
}}
'''.format(long_name)
        data = json.loads(input_json)
        # With truncate='warn' the field should be truncated, but
        # print a warning to stderr:
        importer = PopItImporter(truncate='warn')
        with capture_output() as (out, err):
            importer.import_from_export_json_data(data)
        output = err.getvalue().strip()
        self.assertIn('Warning: truncating Albert', output)
        self.assertIn('Albert to a length of 512', output)
        person = models.Person.objects.get()
        max_length = person._meta.get_field('name').max_length
        truncated_name = long_name[:max_length]
        self.assertEqual(person.name, truncated_name)

    def test_values_in_related_objects_are_truncated(self):
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice",
            "contact_details": [
                {
                    "type": "official-work-email",
                    "value": "ceo@company.example.org"
                }
            ]
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter(truncate='yes')
        importer.import_from_export_json_data(data)
        cd = models.ContactDetail.objects.get()
        self.assertEqual(cd.contact_type, 'official-wor')
        # Also test that they can be updated without creating a
        # duplicate. (This could happen if the implementation uses the
        # overlong version for checking existence, but the truncated
        # version for setting.)
        importer.import_from_export_json_data(data)
        self.assertEqual(models.ContactDetail.objects.count(), 1)
        cd = models.ContactDetail.objects.get()
        self.assertEqual(cd.contact_type, 'official-wor')

    def test_dont_recreate_related_objects(self):
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice",
            "identifiers": [
                {
                    "identifier": "123456789",
                    "scheme": "yournextmp-candidate"
                }
            ],
            "contact_details": [
                {
                    "contact_type": "twitter",
                    "label": "",
                    "note": "",
                    "value": "sometwitterusernameorother"
                }

            ],
            "links": [
                {
                    "note": "homepage",
                    "url": "http://example.com/alice"
                }
            ]
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Person.objects.count(), 1)
        original_person = models.Person.objects.get()
        self.assertEqual(original_person.name, "Alice")
        self.assertEqual(original_person.identifiers.count(), 2)
        original_identifier_a = original_person.identifiers.get(
            scheme='popit-person')
        original_identifier_b = original_person.identifiers.get(
            scheme='yournextmp-candidate')
        self.assertEqual(original_identifier_a.identifier, "a1b2")
        self.assertEqual(original_identifier_b.identifier, "123456789")
        self.assertEqual(original_person.contact_details.count(), 1)
        original_contact_detail = original_person.contact_details.get()

        # Now try importing again, and refetch those objects:
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Person.objects.count(), 1)
        new_person = models.Person.objects.get()
        self.assertEqual(new_person.name, "Alice")
        self.assertEqual(new_person.identifiers.count(), 2)
        new_identifier_a = new_person.identifiers.get(scheme='popit-person')
        new_identifier_b = new_person.identifiers.get(
            scheme='yournextmp-candidate')
        self.assertEqual(new_identifier_a.identifier, "a1b2")
        self.assertEqual(new_identifier_b.identifier, "123456789")
        self.assertEqual(new_person.contact_details.count(), 1)
        new_contact_detail = new_person.contact_details.get()
        # Now check that these objects haven't changed ID:
        self.assertEqual(original_person.id, new_person.id)
        self.assertEqual(original_identifier_a.id, new_identifier_a.id)
        self.assertEqual(original_identifier_b.id, new_identifier_b.id)
        self.assertEqual(original_contact_detail.id, new_contact_detail.id)

    def test_organization_with_identifiers(self):
        input_json = '''
{
    "organizations": [
        {
            "id": "commons",
            "name": "House of Commons",
            "identifiers": [
                {
                    "scheme": "big-db-of-parliaments",
                    "identifier": "uk-commons"
                }
            ]
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Organization.objects.count(), 1)
        organization = models.Organization.objects.get()
        self.assertEqual(organization.name, "House of Commons")
        self.assertEqual(
            organization.identifiers.get(
                scheme='popit-organization').identifier,
            "commons"
        )
        self.assertEqual(
            organization.identifiers.get(
                scheme='big-db-of-parliaments').identifier,
            "uk-commons"
        )

    def test_update_existing_organization(self):
        existing = models.Organization.objects.create(name="The Commons")
        existing.identifiers.create(
            scheme="popit-organization", identifier="commons")
        input_json = '''
{
    "organizations": [
        {
            "id": "commons",
            "name": "House of Commons"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Organization.objects.count(), 1)
        organization = models.Organization.objects.get()
        self.assertEqual(existing, organization)
        self.assertEqual(organization.name, "House of Commons")
        self.assertEqual(
            organization.identifiers.get(
                scheme='popit-organization').identifier,
            "commons"
        )

    def test_organisation_with_inline_area(self):
        input_json = '''
{
    "organizations": [
        {
            "id": "commons",
            "name": "House of Commons",
            "area": {
                "id": "area-1",
                "name": "The United Kingdom of Great Britain and Northern Ireland",
                "identifier": "uk",
                "classification": "country"
            }
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Organization.objects.count(), 1)
        self.assertEqual(models.Area.objects.count(), 1)
        area = models.Area.objects.get()
        self.assertEqual(
            area.name,
            'The United Kingdom of Great Britain and Northern Ireland')
        self.assertEqual(area.identifier, 'uk')
        self.assertEqual(area.classification, 'country')

    def test_organisation_with_external_area(self):
        input_json = '''
{
    "organizations": [
        {
            "id": "commons",
            "name": "House of Commons",
            "area_id": "area-1"
        }
    ],
    "areas": [
        {
            "id": "area-1",
            "name": "The United Kingdom of Great Britain and Northern Ireland",
            "identifier": "uk",
            "classification": "country"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Organization.objects.count(), 1)
        self.assertEqual(models.Area.objects.count(), 1)
        area = models.Area.objects.get()
        self.assertEqual(
            area.name,
            'The United Kingdom of Great Britain and Northern Ireland')
        self.assertEqual(area.identifier, 'uk')
        self.assertEqual(area.classification, 'country')

    def test_post_creation(self):
        input_json = '''
{
    "posts": [
        {
            "id": "65808",
            "label": "Member of Parliament for Dulwich and West Norwood",
            "role": "Member of Parliament",
            "organization_id": "commons"
        }
    ],
    "organizations": [
        {
            "id": "commons",
            "name": "House of Commons"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Post.objects.count(), 1)
        post = models.Post.objects.get()
        self.assertEqual(post.label,
                         "Member of Parliament for Dulwich and West Norwood")
        self.assertEqual(post.role, "Member of Parliament")
        self.assertEqual(models.Organization.objects.count(), 1)

    def test_post_update(self):
        existing_org = models.Organization.objects.create(
            name='House of Commons')
        existing_org.identifiers.create(
            scheme='popit-organization', identifier='commons')
        existing_post = models.Post.objects.create(
            label="MP for Dulwich and West Norwood",
            role="MP",
            organization=existing_org
        )
        models.Identifier.objects.create(
            scheme='popit-post',
            identifier='65808',
            content_object=existing_post,
        )
        # Now import JSON that updates that post:
        input_json = '''
{
    "posts": [
        {
            "id": "65808",
            "label": "Member of Parliament for Dulwich and West Norwood",
            "role": "Member of Parliament",
            "organization_id": "commons"
        }
    ],
    "organizations": [
        {
            "id": "commons",
            "name": "House of Commons"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Post.objects.count(), 1)
        post = models.Post.objects.get()
        self.assertEqual(post.label,
                         "Member of Parliament for Dulwich and West Norwood")
        self.assertEqual(post.role, "Member of Parliament")
        self.assertEqual(models.Organization.objects.count(), 1)

    def test_exception_from_inline_area_missing_id(self):
        input_json = '''
{
    "organizations": [
        {
            "id": "commons",
            "name": "House of Commons",
            "area": {
                "name": "The United Kingdom of Great Britain and Northern Ireland",
                "identifier": "uk",
                "classification": "country"
            }
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        with capture_output() as (out, err):
            with self.assertRaisesRegexp(
                    ValueError,
                    'Found inline area data, but with no "id" attribute'):
                importer.import_from_export_json_data(data)

    def test_organization_parent_relationship(self):
        input_json = '''
{
    "organizations": [
        {
            "id": "co",
            "name": "Cabinet Office",
            "parent_id": "cs"
        },
        {
            "id": "cs",
            "name": "Civil Service"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Organization.objects.count(), 2)
        child = models.Organization.objects.get(name='Cabinet Office')
        parent = models.Organization.objects.get(name='Civil Service')
        self.assertEqual(child.parent, parent)

    def test_area_update(self):
        existing = models.Area.objects.create(
            name="Alba",
            identifier="blah-sco",
            classification="country",
        )
        existing.other_identifiers.create(
            scheme='popit-area',
            identifier='subarea-1',
        )
        input_json = '''
{
    "areas": [
        {
            "id": "subarea-1",
            "name": "Scotland",
            "identifier": "sco",
            "classification": "country"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Area.objects.count(), 1)
        area = models.Area.objects.get()
        self.assertEqual(area.name, 'Scotland')
        self.assertEqual(area.identifier, 'sco')
        self.assertEqual(area.classification, 'country')

    def test_area_parent_relationship(self):
        input_json = '''
{
    "areas": [
        {
            "id": 2,
            "name": "Scotland",
            "identifier": "sco",
            "classification": "country",
            "parent_id": 1
        },
        {
            "id": 1,
            "name": "United Kingdom",
            "identifier": "uk",
            "classification": "country"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Area.objects.count(), 2)
        child = models.Area.objects.get(name='Scotland')
        parent = models.Area.objects.get(name='United Kingdom')
        self.assertEqual(child.parent, parent)

    def test_inline_area_parent_relationship(self):
        input_json = '''
{
    "organizations": [
        {
            "id": "sp",
            "name": "Scottish Parliament",
            "area": {
                "id": "subarea-1",
                "name": "Scotland",
                "identifier": "sco",
                "classification": "country",
                "parent_id": "area-1"
            }
        }
    ],
    "areas": [
        {
            "id": "area-1",
            "name": "United Kingdom",
            "identifier": "uk",
            "classification": "country"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Area.objects.count(), 2)
        child = models.Area.objects.get(name='Scotland')
        parent = models.Area.objects.get(name='United Kingdom')
        self.assertEqual(child.parent, parent)

    def test_link_and_source_without_a_note(self):
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice",
            "sources": [
               {
                   "url": "http://example.org/source-of-alice-data"
               }
            ],
            "links": [
               {
                   "url": "http://example.org/a-link-without-a-note"
               }
            ]
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Person.objects.count(), 1)
        person = models.Person.objects.get()
        self.assertEqual(person.name, "Alice")
        self.assertEqual(
            person.sources.get().url,
            "http://example.org/source-of-alice-data")
        self.assertEqual(
            person.links.get().url,
            "http://example.org/a-link-without-a-note")

    def test_remove_identifiers_even_if_none_specified(self):
        existing_person = models.Person.objects.create(name='Algernon')
        existing_person.identifiers.create(
            scheme='popolo:person',
            identifier="a1b2"
        )
        existing_person.identifiers.create(
            scheme='ignorable',
            identifier="some-data-we-care-nothing-for"
        )
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice"
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter(id_prefix='popolo:')
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Person.objects.count(), 1)
        # Reget the person from the database:
        person = models.Person.objects.get(pk=existing_person.id)
        self.assertEqual(person.name, 'Alice')
        self.assertSequenceEqual(
            person.identifiers.order_by('scheme').values_list('scheme',
                                                              'identifier'),
            [(u'popolo:person', u'a1b2')]
        )

    def test_remove_identifiers_when_empty_array_specified(self):
        existing_person = models.Person.objects.create(name='Algernon')
        existing_person.identifiers.create(
            scheme='popolo:person',
            identifier="a1b2"
        )
        existing_person.identifiers.create(
            scheme='ignorable',
            identifier="some-data-we-care-nothing-for"
        )
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice",
            "identifiers": []
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter(id_prefix='popolo:')
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Person.objects.count(), 1)
        # Reget the person from the database:
        person = models.Person.objects.get(pk=existing_person.id)
        self.assertEqual(person.name, 'Alice')
        self.assertSequenceEqual(
            person.identifiers.order_by('scheme').values_list('scheme',
                                                              'identifier'),
            [(u'popolo:person', u'a1b2')]
        )

    def test_extra_preserved_identifiers(self):
        existing_person = models.Person.objects.create(name='Algernon')
        existing_person.identifiers.create(
            scheme='popolo:person',
            identifier="a1b2"
        )
        existing_person.identifiers.create(
            scheme='ignorable',
            identifier="some-data-we-care-nothing-for"
        )
        existing_person.identifiers.create(
            scheme='preserve-me',
            identifier="data-that-should-be-kept"
        )
        input_json = '''
{
    "persons": [
        {
            "id": "a1b2",
            "name": "Alice",
            "identifiers": []
        }
    ]
}
'''
        data = json.loads(input_json)
        importer = PopItImporter(
            id_prefix='popolo:',
            id_schemes_to_preserve={'person': {'preserve-me'}})
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Person.objects.count(), 1)
        # Reget the person from the database:
        person = models.Person.objects.get(pk=existing_person.id)
        self.assertEqual(person.name, 'Alice')
        self.assertSequenceEqual(
            person.identifiers.order_by('scheme').values_list('scheme',
                                                              'identifier'),
            [(u'popolo:person', u'a1b2'),
             (u'preserve-me', 'data-that-should-be-kept')]
        )


class SurprisingExceptionTests(TestCase):
    def test_exception_unknown_collection_on_get_existing(self):
        importer = PopItImporter()
        with self.assertRaisesRegexp(
                Exception,
                r"Unknown collection 'not-a-collection'"):
            importer.get_existing_django_object('not-a-collection', 'commons')

    def test_exception_unknown_collection_on_create_identifier(self):
        commons = models.Organization.objects.create(name="The Commons")
        importer = PopItImporter()
        with self.assertRaisesRegexp(
                Exception,
                r"Unknown collection 'not-a-collection'"):
            importer.create_identifier('not-a-collection', 'commons', commons)


class LegislativePeriodMembershipTests(TestCase):
    """Tests for a special case to provide defaults for membership dates"""

    def setUp(self):
        # This Input data extracted from:
        # https://cdn.rawgit.com/everypolitician/everypolitician-data
        # /a46ef721903ad86a07af2abbebf469015e9b4367/data/UK/Commons/ep-popolo
        # -v1.0.json
        # This is quite a useful sample of real data so we should
        # reuse this to test the rest of it is parsed properly (FIXME).
        self.input_json = u'''
{
    "persons": [
        {
            "birth_date": "1955-04-09",
            "gender": "male",
            "given_name": "Eric",
            "id": "0588816f-94b5-4397-88f6-f3fdca0f94f0",
            "identifiers": [
                {
                    "identifier": "Commons/member/386",
                    "scheme": "parliamentdotuk"
                }
            ],
            "links": [
                {
                    "note": "Wikipedia (en)",
                    "url": "https://en.wikipedia.org/wiki/Eric_Illsley"
                }
            ],
            "name": "Eric Illsley",
            "other_names": [
                {
                    "lang": "ja",
                    "name": "エリック・イルズリー",
                    "note": "multilingual"
                }
            ]
        },
        {
            "birth_date": "1971-05-23",
            "contact_details": [
                {
                    "type": "email",
                    "value": "robertsonbj@parliament.uk"
                },
                {
                    "type": "email",
                    "value": "george.osborne.mp@parliament.uk"
                },
                {
                    "type": "phone",
                    "value": "020 7219 8214"
                },
                {
                    "type": "twitter",
                    "value": "George_Osborne"
                }
            ],
            "email": "robertsonbj@parliament.uk",
            "gender": "male",
            "given_name": "George",
            "id": "008ca7b1-fd32-420f-b710-d49fe8e34b05",
            "name": "George Osborne",
            "sort_name": "Osborne, Mr George"
        }
    ],
    "memberships": [
        {
            "area_id": "uk.org.publicwhip/cons/22",
            "end_date": "2011-02-08",
            "legislative_period_id": "term/55",
            "on_behalf_of_id": "labour",
            "organization_id": "8bbf6031-aa59-4b0a-80b2-1cdc81e4047d",
            "person_id": "0588816f-94b5-4397-88f6-f3fdca0f94f0",
            "role": "member"
        },
        {
            "area_id": "uk.org.publicwhip/cons/572",
            "legislative_period_id": "term/56",
            "on_behalf_of_id": "conservative",
            "organization_id": "8bbf6031-aa59-4b0a-80b2-1cdc81e4047d",
            "person_id": "008ca7b1-fd32-420f-b710-d49fe8e34b05",
            "role": "member"
        }
    ],
    "events": [
        {
            "classification": "legislative period",
            "end_date": "2015-03-30",
            "id": "term/55",
            "name": "55th Parliament of the United Kingdom",
            "organization_id": "8bbf6031-aa59-4b0a-80b2-1cdc81e4047d",
            "start_date": "2010-05-06",
            "wikidata": "Q21084472"
        },
        {
            "classification": "legislative period",
            "id": "term/56",
            "name": "56th Parliament of the United Kingdom",
            "organization_id": "8bbf6031-aa59-4b0a-80b2-1cdc81e4047d",
            "start_date": "2015-05-08",
            "wikidata": "Q21084473"
        }
    ],
    "organizations": [
        {
            "classification": "legislature",
            "id": "8bbf6031-aa59-4b0a-80b2-1cdc81e4047d",
            "identifiers": [
                {
                    "identifier": "Q11005",
                    "scheme": "wikidata"
                }
            ],
            "name": "House of Commons",
            "seats": 650
        },
        {
            "classification": "party",
            "id": "labour",
            "identifiers": [
                {
                    "identifier": "Q9630",
                    "scheme": "wikidata"
                }
            ],
            "name": "Labour"
        },
        {
            "classification": "party",
            "id": "conservative",
            "identifiers": [
                {
                    "identifier": "Q9626",
                    "scheme": "wikidata"
                }
            ],
            "links": [
                {
                    "note": "website",
                    "url": "http://www.conservatives.com"
                }
            ],
            "name": "Conservative"
        }
    ]
}
        '''

    def test_membership_no_dates_but_some_in_legislative_period(self):
        data = json.loads(self.input_json)
        importer = PopItImporter()
        importer.import_from_export_json_data(data)
        self.assertEqual(models.Membership.objects.count(), 2)
        earlier, later = models.Membership.objects.order_by('start_date')
        self.assertEqual(earlier.start_date, '2010-05-06')
        self.assertEqual(earlier.end_date, '2011-02-08')
        self.assertEqual(later.start_date, '2015-05-08')
        self.assertEqual(later.end_date, '')


class PopItSubclassTests(TestCase):
    def test_error_on_wrong_arguments(self):
        with self.assertRaises(Exception):
            call_command('popolo_create_from_popit')

    def test_special_cases_for_popit(self):
        # 1536 characters long; longer than the 1024 max_length
        overlong_summary = 'abcdefgh' * 192
        # Make sure the URL exceeds 200 characters
        overlong_url = 'http://www.parliament.uk/business/commons/#'
        overlong_url += '0123' * 100
        input_json = '''
{{
    "persons": [
        {{
            "id": "a1b2",
            "name": "Alice",
            "email": " alice @ example.org  ",
            "summary": "{}",
            "contact_details": [
                {{
                    "type": "official-work-email",
                    "value": "ceo@company.example.org"
                }}
            ]
        }}
    ],
    "organizations": [
        {{
            "id": "commons",
            "name": "House of Commons",
            "links": [
                {{
                    "note": "website",
                    "url": "{}"
                }}
            ]
        }}
    ],
    "memberships": [
        {{
            "person_id": "a1b2",
            "organization_id": "commons"
        }}
    ]
}}
'''.format(overlong_summary, overlong_url)
        try:
            with NamedTemporaryFile('w', delete=False, suffix='.json') as ntf:
                ntf.write(input_json)
            call_command('popolo_create_from_popit', ntf.name)
        finally:
            os.remove(ntf.name)
        # Now check everything was created:
        self.assertEqual(models.Membership.objects.count(), 1)
        self.assertEqual(models.Person.objects.count(), 1)
        self.assertEqual(models.Organization.objects.count(), 1)
        self.assertEqual(models.Identifier.objects.count(), 3)
        self.assertEqual(models.ContactDetail.objects.count(), 1)
        # Check that the bad properties have been fixed as expected:
        person = models.Person.objects.get()
        self.assertEqual(person.email, 'alice@example.org')
        self.assertEqual(len(person.summary), 1024)
        contact = models.ContactDetail.objects.get()
        self.assertEqual(contact.contact_type, 'official-wor')
        self.assertEqual(contact.value, 'ceo@company.example.org')
        link = models.Link.objects.get()
        self.assertEqual(link.content_object, models.Organization.objects.get())
        self.assertEqual(link.note, 'website')
        self.assertEqual(len(link.url), 350)
        self.assertTrue(
            link.url.startswith(
                'http://www.parliament.uk/business/commons/#01230123'))



from time import sleep
from datetime import timedelta
from datetime import datetime
from django.core.exceptions import ValidationError


class BehaviorTestCaseMixin(object):
    def get_model(self):
        return getattr(self, 'model')

    def create_instance(self, **kwargs):
        raise NotImplementedError("Implement me")


class DateframeableTests(BehaviorTestCaseMixin):
    """
    Dateframeable tests.

    Are dates valid? Are invalid dates blocked?
    Are querysets to filter past, present and future items correct?
    """

    def test_new_instance_has_valid_dates(self):
        """Test complete or incomplete dates,
        according to the "^[0-9]{4}(-[0-9]{2}){0,2}$" pattern (incomplete
        dates)"""
        obj = self.create_instance(start_date='2012')
        self.assertRegexpMatches(obj.start_date, "^[0-9]{4}(-[0-9]{2}){0,2}$",
                                 "date does not match pattern")
        obj = self.create_instance(end_date='2012')
        self.assertRegexpMatches(obj.end_date, "^[0-9]{4}(-[0-9]{2}){0,2}$",
                                 "date does not match pattern")

        obj = self.create_instance(start_date='2012-01')
        self.assertRegexpMatches(obj.start_date, "^[0-9]{4}(-[0-9]{2}){0,2}$",
                                 "date does not match pattern")
        obj = self.create_instance(end_date='2012-02')
        self.assertRegexpMatches(obj.end_date, "^[0-9]{4}(-[0-9]{2}){0,2}$",
                                 "date does not match pattern")

        obj = self.create_instance(start_date='2012-10-12')
        self.assertRegexpMatches(obj.start_date, "^[0-9]{4}(-[0-9]{2}){0,2}$",
                                 "date does not match pattern")
        obj = self.create_instance(end_date='2012-12-10')
        self.assertRegexpMatches(obj.end_date, "^[0-9]{4}(-[0-9]{2}){0,2}$",
                                 "date does not match pattern")

    def test_invalid_dates_are_blocked(self):
        """Test if dates are valid (months and days range are tested)"""
        # test invalid start dates
        with self.assertRaises(ValidationError):
            obj = self.create_instance(start_date='YESTERDAY')

        with self.assertRaises(ValidationError):
            obj = self.create_instance(start_date='2012-1210')

        with self.assertRaises(ValidationError):
            obj = self.create_instance(start_date='2012-13')

        with self.assertRaises(ValidationError):
            obj = self.create_instance(start_date='2012-12-34')

        # test invalid end dates
        with self.assertRaises(ValidationError):
            obj = self.create_instance(end_date='YESTERDAY')

        with self.assertRaises(ValidationError):
            obj = self.create_instance(end_date='2012-1210')

        with self.assertRaises(ValidationError):
            obj = self.create_instance(end_date='2012-13')

        with self.assertRaises(ValidationError):
            obj = self.create_instance(end_date='2012-12-34')

    def test_querysets_filters(self):
        """Test current, past and future querysets"""
        past_obj = self.create_instance(
            start_date=datetime.strftime(datetime.now() - timedelta(days=10),
                                         '%Y-%m-%d'),
            end_date=datetime.strftime(datetime.now() - timedelta(days=5),
                                       '%Y-%m-%d'))
        current_obj = self.create_instance(
            start_date=datetime.strftime(datetime.now() - timedelta(days=5),
                                         '%Y-%m-%d'),
            end_date=datetime.strftime(datetime.now() + timedelta(days=5),
                                       '%Y-%m-%d'))
        future_obj = self.create_instance(
            start_date=datetime.strftime(datetime.now() + timedelta(days=5),
                                         '%Y-%m-%d'),
            end_date=datetime.strftime(datetime.now() + timedelta(days=10),
                                       '%Y-%m-%d'))

        self.assertEqual(self.get_model().objects.all().count(), 3,
                         "Something really bad is going on")
        self.assertEqual(self.get_model().objects.past().count(), 1,
                         "One past object should have been fetched")
        self.assertEqual(self.get_model().objects.current().count(), 1,
                         "One current object should have been fetched")
        self.assertEqual(self.get_model().objects.future().count(), 1,
                         "One future object should have been fetched")


class TimestampableTests(BehaviorTestCaseMixin):
    """
    Timestampable tests.

    Tests whether objects are assigned timestamps at creation time, and
    whether a successive modification changes the update timestamp only.
    """

    def test_new_instance_has_equal_timestamps(self):
        """Object is assigned timestamps when created"""
        obj = self.create_instance()
        self.assertIsNotNone(obj.created_at)
        self.assertIsNotNone(obj.updated_at)

        # created_at and updated_at are actually different, but still within
        # 2 millisec
        # that's because of the pre-save signal validation
        self.assertTrue(
            (obj.updated_at - obj.created_at) < timedelta(microseconds=10000))

    def test_updated_instance_has_different_timestamps(self):
        """Modified object has different created_at and updated_at timestamps
        """
        obj = self.create_instance()
        creation_ts = obj.created_at
        update_ts = obj.updated_at
        # save object after 30K microsecs and check again
        sleep(0.03)
        obj.save()
        self.assertEqual(obj.created_at, creation_ts)
        self.assertNotEqual(obj.updated_at, update_ts)

        # created_at and updated_at are actually different, well outside 10
        # millisecs
        self.assertFalse(
            (obj.updated_at - obj.created_at) < timedelta(microseconds=10000))


class PermalinkableTests(BehaviorTestCaseMixin):
    """
    Permalinkable tests.

    Tests whether objects are assigned the correct slugs and have the correct absolute_url
    """

    def test_instance_has_slug(self):
        """The instance has the correct slug (all empty instances have the test-instance slug, by default)"""
        i = self.create_instance()
        self.assertEqual(i.slug, 'test-instance')

    def test_instance_has_permalink(self):
        i = self.create_instance()
        self.assertEqual(i.get_absolute_url(),
                         '/{0}/{1}/'.format(self.object_name, i.slug))



try:
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    # This fallback import is the version that was deprecated in
    # Django 1.7 and is removed in 1.9:
    from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.db import models
from django.utils.translation import ugettext_lazy as _
from model_utils.fields import AutoCreatedField, AutoLastModifiedField
from autoslug import AutoSlugField
from datetime import datetime

__author__ = 'guglielmo'


def get_slug_source(instance):
    """For use in AutoSlugField's populate_from"""
    return instance.slug_source


class GenericRelatable(models.Model):
    """
    An abstract class that provides the possibility of generic relations
    """
    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    object_id = models.PositiveIntegerField(null=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    class Meta:
        abstract = True


def validate_partial_date(value):
    """
    Validate a partial date, it can be partial, but it must yet be a valid date.
    Accepted formats are: YYYY-MM-DD, YYYY-MM, YYYY.
    2013-22 must rais a ValidationError, as 2013-13-12, or 2013-11-55.
    """
    try:
        datetime.strptime(value, '%Y-%m-%d')
    except ValueError:
        try:
            datetime.strptime(value, '%Y-%m')
        except ValueError:
            try:
                datetime.strptime(value, '%Y')
            except ValueError:
                raise ValidationError(
                    u'date seems not to be correct %s' % value)


class Dateframeable(models.Model):
    """
    An abstract base class model that provides a start and an end dates to
    the class.
    Uncomplete dates can be used. The validation pattern is: "^[0-9]{4}(-[
    0-9]{2}){0,2}$"
    """
    partial_date_validator = RegexValidator(regex="^[0-9]{4}(-[0-9]{2}){0,2}$",
                                            message="Date has wrong format")

    start_date = models.CharField(
        _("start date"), max_length=10, blank=True, null=True,
        validators=[partial_date_validator, validate_partial_date],
        help_text=_("The date when the validity of the item starts"),
    )
    end_date = models.CharField(
        _("end date"), max_length=10, blank=True, null=True,
        validators=[partial_date_validator, validate_partial_date],
        help_text=_("The date when the validity of the item ends")
    )

    class Meta:
        abstract = True


class Timestampable(models.Model):
    """
    An abstract base class model that provides self-updating
    ``created`` and ``modified`` fields.
    """
    created_at = AutoCreatedField(_('creation time'))
    updated_at = AutoLastModifiedField(_('last modification time'))

    class Meta:
        abstract = True


class Permalinkable(models.Model):
    """
    An abstract base class model that provides a unique slug,
    and the methods necessary to handle the permalink
    """
    from django.utils.text import slugify

    slug = AutoSlugField(
        populate_from=get_slug_source,
        max_length=255,
        unique=True,
        slugify=slugify
    )

    class Meta:
        abstract = True

    def get_url_kwargs(self, **kwargs):
        kwargs.update(getattr(self, 'url_kwargs', {}))
        return kwargs

    @models.permalink
    def get_absolute_url(self):
        url_kwargs = self.get_url_kwargs(slug=self.slug)
        return (self.url_name, (), url_kwargs)



import os
from setuptools import setup, find_packages


def read_file(filename):
    """Read a file into a string"""
    path = os.path.abspath(os.path.dirname(__file__))
    filepath = os.path.join(path, filename)
    try:
        return open(filepath).read()
    except IOError:
        return ''


setup(
    name='django-popolo',
    version=__import__('popolo').__version__,
    author='Guglielmo Celata',
    author_email='guglielmo@openpolis.it',
    packages=find_packages(),
    include_package_data=True,
    url='http://github.com/openpolis/django-popolo',
    license='Affero',
    description=u' '.join(__import__('popolo').__doc__.splitlines()).strip(),
    classifiers=[
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Django',
        'Development Status :: 5 - Production/Stable',
        'Operating System :: OS Independent',
    ],
    long_description=read_file('README.rst'),
    test_suite="runtests.runtests",
    zip_safe=False,
    tests_require=[
        'Django',
        'fake-factory',
        'mock==2.0.0'
    ],
    install_requires=[
        "django-autoslug",
        "django-model-utils",
    ],
)



from __future__ import print_function

import re

from django.core.management.base import BaseCommand, CommandError

from popolo.importers.popolo_json import PopoloJSONImporter


class Command(PopoloJSONImporter, BaseCommand):
    def __init__(self, *args, **kwargs):
        # django-popolo has restricted lengths for various fields,
        # whereas PopIt has no such aritrary limits; so, for the
        # import not to fail, we need to truncate them.
        kwargs['truncate'] = 'yes'
        super(Command, self).__init__(*args, **kwargs)

    # This is so that this command works on Django 1.8 as well as
    # earlier versions. See "Changed in Django 1.8" here:
    # https://docs.djangoproject.com/en/1.8/howto/custom-management-commands/
    def add_arguments(self, parser):
        parser.add_argument('args', nargs='+')

    def handle(self, *args, **options):

        if len(args) != 1:
            message = "You must supply a filename with exported PopIt JSON"
            raise CommandError(message)

        popit_export_filename = args[0]

        self.import_from_export_json(popit_export_filename)

    # ------------------------------------------------------------------------
    # This overridden method deals with an awkward incompatability
    # between what PopIt and django-popolo allow.

    def update_person(self, person_data):
        new_person_data = person_data.copy()
        # Surprisingly, quite a lot of PopIt email addresses have
        # extraneous whitespace in them, so strip any out to avoid
        # the 'Enter a valid email address' ValidationError on saving:
        email = person_data.get('email') or None
        if email:
            email = re.sub(r'\s*', '', email)
        new_person_data['email'] = email
        return super(Command, self).update_person(new_person_data)



from __future__ import print_function

from contextlib import contextmanager
import json
import sys

from django.utils.six import text_type

from django.apps import apps

NEW_COLLECTIONS = ('organization', 'post', 'person', 'membership', 'area')


@contextmanager
def show_data_on_error(variable_name, data):
    """A context manager to output problematic data on any exception

    If there's an error when importing a particular person, says, it's
    useful to have in the error output that particular structure that
    caused problems. If you wrap the code that processes some data
    structure (a dictionary called 'my_data', say) with this:

        with show_data_on_error('my_data', my_data'):
            ...
            process(my_data)
            ...

    ... then if any exception is thrown in the 'with' block you'll see
    the data that was being processed when it was thrown."""

    try:
        yield
    except:
        message = 'An exception was thrown while processing {0}:'
        print(message.format(variable_name), file=sys.stderr)
        print(json.dumps(data, indent=4, sort_keys=True))
        raise


class PopoloJSONImporter(object):
    """This class helps you to import data from Popolo JSON into django-popolo

    You can use an intance of this class to import data from a Popolo
    JSON file to django-popolo models.  This is useful, for example,
    for importing data from EveryPolitician or from PopIt instances
    (using the /api/v0.1/export.json API endpoint).

    This importer will create Identifier objects that let you access
    the instances of Django models via their IDs in the source data.
    You can customize the prefix used in the scheme of the Identifier
    objects with the id_prefix argument to this class's initializer.
    e.g. if you use id_prefix="popolo:" then each Person will have an
    Identifier with scheme "popolo:person" with their ID from the
    source.  (The default for id_prefix is "popit-".)

    This is designed to be easy to subclass to extend its
    behaviour. For example:

    * If you need to use this importer in a migration, you can
      override the initializer and get_popolo_model_class to return
      the unfrozen model classes instead of the real current model
      classes.

    * If you need to preprocess the data being added you can override
      the methods that transform PopIt to django-popolo data (e.g. you
      could override make_link_dict to truncate excessively long
      URLs.)

    * If you're using multi-table inheritance or explicit one-to-one
      fields to add extra attributes to the django-popolo models, then
      you can add that data by overriding one or more of
      update_person, update_membership, etc.

    """

    TRUNCATE_OPTIONS = {'yes', 'warn', 'exception'}

    def __init__(self, id_prefix=None, truncate='exception', **kwargs):
        super(PopoloJSONImporter, self).__init__()
        if id_prefix is None:
            self.id_prefix = 'popit-'
        else:
            self.id_prefix = id_prefix
        if truncate not in PopoloJSONImporter.TRUNCATE_OPTIONS:
            msg = "Unknown option for truncate '{0}'; it must be one of: {1}"
            raise ValueError(msg.format(
                truncate, PopoloJSONImporter.TRUNCATE_OPTIONS))
        self.truncate = truncate
        # By default we preserve the ID schemes used to identify
        # previously imported data:
        self.id_schemes_to_preserve = {
            'person': {self.id_prefix + 'person'},
            'organization': {self.id_prefix + 'organization'},
            'area': {self.id_prefix + 'area'},
        }
        # And if any extra ID schemes have been requested for
        # preservation, keep them too:
        if kwargs.get('id_schemes_to_preserve'):
            self.add_id_schemes_to_preserve(kwargs['id_schemes_to_preserve'])
        self.observers = []

    def add_id_schemes_to_preserve(self, id_schemes_to_preserve):
        for collection, schemes in id_schemes_to_preserve.items():
            if collection not in NEW_COLLECTIONS:
                raise Exception("Unknown collection: '{}'".format(collection))
            self.id_schemes_to_preserve.setdefault(collection, {})
            self.id_schemes_to_preserve[collection].update(schemes)

    def add_observer(self, observer):
        self.observers.append(observer)

    def notify_observers(self, collection, django_object, created, popolo_data):
        for o in self.observers:
            o.notify(collection, django_object, created, popolo_data)

    def get_popolo_model_class(self, model_name):
        """A default implementation for getting the Popolo model class"""
        return self.get_model_class('popolo', model_name)

    def get_model_class(self, app_label, model_name):
        return apps.get_model(app_label, model_name)

    def get_truncated(self, django_object_or_model_class, field_name, value):
        meta = django_object_or_model_class._meta
        max_length = meta.get_field(field_name).max_length
        # If there's no max_length on the field, just set the value
        if max_length is not None:
            # If an exception is wanted, we just try setting the value as normal
            if self.truncate != 'exception':
                if value:
                    if self.truncate == 'warn' and len(value) > max_length:
                        msg = "Warning: truncating {0} to a length of {1}"
                        print(msg.format(value, max_length), file=sys.stderr)
                    value = value[:max_length]
        return value

    def set(self, django_object, field_name, value):
        possibly_truncated_value = self.get_truncated(
            django_object, field_name, value)
        setattr(django_object, field_name, possibly_truncated_value)

    def import_from_export_json_data(self, data):
        """Update or create django-popolo models from a PopIt export

        You can run this multiple times to update the django-popolo
        models after the initial import. This version of the method takes
        the JSON data after parsing; i.e. as Python lists, dicts, etc."""

        # Keep track of all areas that are found, so that we can later
        # iterate over them and make sure their 'parent' property is
        # correctly set.
        area_id_to_django_object = {}
        area_id_to_parent_area_id = {}

        def update_optional_area(object_data):
            area_data = object_data.get('area')
            area_id = object_data.get('area_id')
            area = None
            if area_data:
                if not area_data.get('id'):
                    msg = 'Found inline area data, but with no "id" attribute'
                    raise ValueError(msg)
                with show_data_on_error('area_data', area_data):
                    area_id, area = self.update_area(area_data)
                    area_id_to_django_object[area_id] = area
                area_parent_id = area_data.get('parent_id')
                if area_parent_id:
                    area_id_to_parent_area_id[area_id] = area_parent_id
            elif area_id:
                # if we have an area_id instead of an inline area
                area = self.get_existing_django_object('area', area_id)
            return area

        self.events = {}
        # store events in a temp local dict:
        for event_data in data.get('events', []):
            self.events[event_data['id']] = event_data

        # Create all areas:
        for area_data in data.get('areas', []):
            with show_data_on_error('area_data', area_data):
                area_id, area = self.update_area(area_data)
                area_parent_id = area_data.get('parent_id')
                if area_parent_id:
                    area_id_to_parent_area_id[area_id] = area_parent_id
                area_id_to_django_object[area_id] = area

        # Do one pass through the organizations:
        org_id_to_django_object = {}
        for org_data in data.get('organizations', []):
            with show_data_on_error('org_data', org_data):
                area = update_optional_area(org_data)
                popit_id, organization = self.update_organization(org_data,
                                                                  area)
                org_id_to_django_object[popit_id] = organization
        # Then go through the organizations again to set the parent
        # organization:
        for org_data in data.get('organizations', []):
            with show_data_on_error('org_data', org_data):
                org = org_id_to_django_object[org_data['id']]
                parent_id = org_data.get('parent_id')
                if parent_id:
                    org_parent = org_id_to_django_object[parent_id]
                    self.set(org, 'parent', org_parent)
                    org.save()
        # Create all posts (dependent on organizations already existing)
        post_id_to_django_object = {}
        for post_data in data.get('posts', []):
            with show_data_on_error('post_data', post_data):
                area = update_optional_area(post_data)
                popit_id, post = \
                    self.update_post(post_data, area, org_id_to_django_object)
                post_id_to_django_object[popit_id] = post
        # Create all people:
        person_id_to_django_object = {}
        inline_memberships = []
        for person_data in data.get('persons', []):
            with show_data_on_error('person_data', person_data):
                inline_memberships += person_data.pop('memberships', [])
                popit_id, person = \
                    self.update_person(person_data)
                person_id_to_django_object[popit_id] = person
        # Now create all memberships to tie the people, organizations
        # and posts together:
        membership_id_to_django_object = {}
        all_memberships = data.get('memberships', []) + inline_memberships
        for membership_data in all_memberships:
            with show_data_on_error('membership_data', membership_data):
                area = update_optional_area(membership_data)
                membership_id, membership = \
                    self.update_membership(
                        membership_data,
                        area,
                        org_id_to_django_object,
                        post_id_to_django_object,
                        person_id_to_django_object,
                    )
                membership_id_to_django_object[membership_id] = membership

        # Finally set any parent area relationships on areas:
        for area_id, parent_area_id in area_id_to_parent_area_id.items():
            area = area_id_to_django_object[area_id]
            parent_area = area_id_to_django_object[parent_area_id]
            self.set(area, 'parent', parent_area)
            area.save()

    def import_from_export_json(self, json_filename):
        """Update or create django-popolo models from a PopIt export

        You can run this multiple times to update the django-popolo
        models after the initial import. This version of the method takes
        a filename"""

        with open(json_filename) as f:
            data = json.load(f)

        self.import_from_export_json_data(data)

    def get_existing_django_object(self, popit_collection, popit_id):
        Identifier = self.get_popolo_model_class('Identifier')
        if popit_collection not in NEW_COLLECTIONS:
            raise Exception("Unknown collection '{collection}'".format(
                collection=popit_collection
            ))
        try:
            i = Identifier.objects.get(
                scheme=(self.id_prefix + popit_collection),
                identifier=popit_id
            )
            # Following i.content_object doesn't work in a migration, so use
            # a slightly more long-winded way to find the referenced object:
            model_class = self.get_popolo_model_class(i.content_type.model)
            return model_class.objects.get(pk=i.object_id)
        except Identifier.DoesNotExist:
            return None

    def update_organization(self, org_data, area):
        Organization = self.get_popolo_model_class('Organization')
        existing = self.get_existing_django_object('organization',
                                                   org_data['id'])
        if existing is None:
            result = Organization()
        else:
            result = existing
        self.set(result, 'name', org_data['name'])
        self.set(result, 'classification', org_data.get('classification', ''))
        self.set(result, 'dissolution_date',
                 org_data.get('dissolution_date', ''))
        self.set(result, 'founding_date', org_data.get('founding_date', ''))
        self.set(result, 'image', org_data.get('image') or None)
        self.set(result, 'area', area)
        result.save()
        # Create an identifier with the PopIt ID:
        if not existing:
            self.create_identifier('organization', org_data['id'], result)

        # Update other identifiers:
        self.update_related_objects(
            Organization,
            self.get_popolo_model_class('Identifier'),
            self.make_identifier_dict,
            org_data.get('identifiers', []),
            result,
        )
        # Update contact details:
        self.update_related_objects(
            Organization,
            self.get_popolo_model_class('ContactDetail'),
            self.make_contact_detail_dict,
            org_data.get('contact_details', []),
            result
        )
        # Update links:
        self.update_related_objects(
            Organization,
            self.get_popolo_model_class('Link'),
            self.make_link_dict,
            org_data.get('links', []),
            result
        )
        # Update sources:
        self.update_related_objects(
            Organization,
            self.get_popolo_model_class('Source'),
            self.make_source_dict,
            org_data.get('sources', []),
            result
        )
        # Update other names:
        self.update_related_objects(
            Organization,
            self.get_popolo_model_class('OtherName'),
            self.make_other_name_dict,
            org_data.get('other_names', []),
            result
        )
        self.notify_observers('organization', result, existing is None,
                              org_data)
        return org_data['id'], result

    def update_post(self, post_data, area, org_id_to_django_object):
        Post = self.get_popolo_model_class('Post')
        existing = self.get_existing_django_object('post', post_data['id'])
        if existing is None:
            result = Post()
        else:
            result = existing
        self.set(result, 'label', post_data['label'])
        self.set(result, 'role', post_data['role'])
        self.set(result, 'organization',
                 org_id_to_django_object[post_data['organization_id']])
        self.set(result, 'area', area)
        result.save()
        # Create an identifier with the PopIt ID:
        if not existing:
            self.create_identifier('post', post_data['id'], result)
        # Update contact details:
        self.update_related_objects(
            Post,
            self.get_popolo_model_class('ContactDetail'),
            self.make_contact_detail_dict,
            post_data.get('contact_details', []),
            result
        )
        # Update links:
        self.update_related_objects(
            Post,
            self.get_popolo_model_class('Link'),
            self.make_link_dict,
            post_data.get('links', []),
            result
        )
        # Update sources:
        self.update_related_objects(
            Post,
            self.get_popolo_model_class('Source'),
            self.make_source_dict,
            post_data.get('sources', []),
            result
        )
        self.notify_observers('post', result, existing is None, post_data)
        return post_data['id'], result

    def update_person(self, person_data):
        Person = self.get_popolo_model_class('Person')
        existing = self.get_existing_django_object('person', person_data['id'])
        if existing is None:
            result = Person()
        else:
            result = existing
        self.set(result, 'name', person_data['name'])
        self.set(result, 'family_name', person_data.get('family_name') or '')
        self.set(result, 'given_name', person_data.get('given_name') or '')
        self.set(result, 'additional_name',
                 person_data.get('additional_name') or '')
        self.set(result, 'honorific_prefix',
                 person_data.get('honorific_prefix') or '')
        self.set(result, 'honorific_suffix',
                 person_data.get('honorific_suffix') or '')
        self.set(result, 'patronymic_name',
                 person_data.get('patronymic_name') or '')
        self.set(result, 'sort_name', person_data.get('sort_name') or '')
        self.set(result, 'email', person_data.get('email') or None)
        self.set(result, 'gender', person_data.get('gender') or '')
        self.set(result, 'birth_date', person_data.get('birth_date') or '')
        self.set(result, 'death_date', person_data.get('death_date') or '')
        self.set(result, 'summary', person_data.get('summary') or '')
        self.set(result, 'biography', person_data.get('biography') or '')
        self.set(result, 'national_identity',
                 person_data.get('national_identity') or None)
        self.set(result, 'image', person_data.get('image') or None)
        result.save()
        # Create an identifier with the PopIt ID:
        if not existing:
            self.create_identifier('person', person_data['id'], result)

        # Update other_names:
        self.update_related_objects(
            Person,
            self.get_popolo_model_class('OtherName'),
            self.make_other_name_dict,
            person_data.get('other_names', []),
            result
        )
        # Update other identifiers:
        self.update_related_objects(
            Person,
            self.get_popolo_model_class('Identifier'),
            self.make_identifier_dict,
            person_data.get('identifiers', []),
            result,
        )
        # Update contact details:
        self.update_related_objects(
            Person,
            self.get_popolo_model_class('ContactDetail'),
            self.make_contact_detail_dict,
            person_data.get('contact_details', []),
            result
        )
        # Update links:
        self.update_related_objects(
            Person,
            self.get_popolo_model_class('Link'),
            self.make_link_dict,
            person_data.get('links', []),
            result
        )
        # Update sources:
        self.update_related_objects(
            Person,
            self.get_popolo_model_class('Source'),
            self.make_source_dict,
            person_data.get('sources', []),
            result
        )
        self.notify_observers('person', result, existing is None, person_data)
        return person_data['id'], result

    def update_membership(
            self,
            membership_data,
            area,
            org_id_to_django_object,
            post_id_to_django_object,
            person_id_to_django_object,
    ):
        def generate_membership_id(membership_data):
            # construct an 'id' based on data that should be unique_together
            new_id = ''
            new_id += membership_data.get('legislative_period_id',
                                          'missing') + "_"
            new_id += membership_data.get('organization_id', 'missing') + "_"
            new_id += membership_data.get('area_id', 'missing') + "_"
            new_id += membership_data.get('role', 'missing') + "_"
            new_id += membership_data.get('on_behalf_of_id', 'missing') + "_"
            new_id += membership_data.get('person_id', 'missing')
            return new_id

        Membership = self.get_popolo_model_class('Membership')

        if 'id' not in membership_data:
            membership_data['id'] = generate_membership_id(membership_data)

        existing = self.get_existing_django_object('membership',
                                                   membership_data['id'])
        if existing is None:
            result = Membership()
        else:
            result = existing
        self.set(result, 'label', membership_data.get('label') or '')
        self.set(result, 'role', membership_data.get('role') or '')
        self.set(result, 'person',
                 person_id_to_django_object[membership_data['person_id']])
        organization_id = membership_data.get('organization_id')
        if organization_id:
            self.set(result, 'organization',
                     org_id_to_django_object[organization_id])
        on_behalf_of_id = membership_data.get('on_behalf_of_id')
        if on_behalf_of_id:
            self.set(result, 'on_behalf_of',
                     org_id_to_django_object[on_behalf_of_id])
        post_id = membership_data.get('post_id')
        if post_id:
            self.set(result, 'post', post_id_to_django_object[post_id])
        self.set(result, 'area', area)
        self.set(result, 'start_date', membership_data.get('start_date', ''))
        self.set(result, 'end_date', membership_data.get('end_date', ''))
        if 'legislative_period_id' in membership_data:
            # Strictly speaking there may be cases where it would not
            # be correct to use the leglislative period's start and
            # end dates as defaults for the membership start and end
            # dates. Ideally, django-popolo would support Popolo
            # events, so we could just associate the membership with
            # one of them and omit the start / end dates as in the
            # original data. However, until events have been added
            # this is a useful default for people importing data where
            # memberships have a legislative period but no start / end
            # date.
            period_data = self.events[membership_data['legislative_period_id']]
            if not result.start_date:
                self.set(result, 'start_date',
                         period_data.get('start_date', ''))
            if not result.end_date:
                self.set(result, 'end_date', period_data.get('end_date', ''))

        result.save()
        # Create an identifier with the PopIt ID:
        if not existing:
            self.create_identifier('membership', membership_data['id'], result)

        # Update contact details:
        self.update_related_objects(
            Membership,
            self.get_popolo_model_class('ContactDetail'),
            self.make_contact_detail_dict,
            membership_data.get('contact_details', []),
            result
        )
        # Update links:
        self.update_related_objects(
            Membership,
            self.get_popolo_model_class('Link'),
            self.make_link_dict,
            membership_data.get('links', []),
            result
        )
        # Update sources:
        self.update_related_objects(
            Membership,
            self.get_popolo_model_class('Source'),
            self.make_source_dict,
            membership_data.get('sources', []),
            result
        )
        self.notify_observers('membership', result, existing is None,
                              membership_data)
        return membership_data['id'], result

    def update_area(self, area_data):
        Area = self.get_popolo_model_class('Area')
        existing = self.get_existing_django_object('area', area_data['id'])
        if existing is None:
            result = Area()
        else:
            result = existing
        self.set(result, 'name', area_data.get('name') or '')
        self.set(result, 'identifier', area_data.get('identifier') or '')
        self.set(result, 'classification',
                 area_data.get('classification') or '')
        self.set(result, 'geom', area_data.get('geom') or None)
        self.set(result, 'inhabitants', area_data.get('inhabitants'))
        result.save()
        # Create an identifier with the PopIt ID:
        if not existing:
            self.create_identifier('area', area_data['id'], result)
        # Update other_identifiers:
        self.update_related_objects(
            Area,
            self.get_popolo_model_class('Identifier'),
            self.make_identifier_dict,
            area_data.get('other_identifiers', []),
            result,
        )
        # Update sources:
        self.update_related_objects(
            Area,
            self.get_popolo_model_class('Source'),
            self.make_source_dict,
            area_data.get('sources', []),
            result
        )
        self.notify_observers('area', result, existing is None, area_data)
        return area_data['id'], result

    def create_identifier(self, popit_collection, popit_id, django_object):
        if popit_collection not in NEW_COLLECTIONS:
            raise Exception("Unknown collection '{collection}'".format(
                collection=popit_collection
            ))
        ContentType = self.get_model_class('contenttypes', 'ContentType')
        content_type = ContentType.objects.get_for_model(django_object)
        self.get_popolo_model_class('Identifier').objects.create(
            object_id=django_object.id,
            content_type_id=content_type.id,
            scheme=(self.id_prefix + popit_collection),
            identifier=popit_id,
        )

    def should_preserve_related(self, django_main_model, related_object):
        # We only have rules for preserving identifiers, so ignore
        # other types of related object:
        if type(related_object).__name__ == 'Identifier':
            schemes_to_preserve = self.id_schemes_to_preserve.get(
                django_main_model.__name__.lower(), set())
            if related_object.scheme in schemes_to_preserve:
                return True

    def update_related_objects(
            self,
            django_main_model,
            django_related_model,
            popit_to_django_attributes_method,
            popit_related_objects_data,
            django_object,
    ):
        # Find the unchanged related objects so we don't unnecessarily
        # recreate objects.
        ContentType = self.get_model_class('contenttypes', 'ContentType')
        main_content_type = ContentType.objects.get_for_model(django_main_model)
        new_objects = []
        old_objects_to_preserve = [
            o for o in django_related_model.objects.filter(
                content_type_id=main_content_type.id,
                object_id=django_object.id
            )
            if self.should_preserve_related(django_main_model, o)
        ]
        for object_data in popit_related_objects_data:
            wanted_attributes = popit_to_django_attributes_method(
                object_data
            )
            # We might need to truncate these in order for them to
            # match the objects already in the database:
            wanted_attributes = {
                k: self.get_truncated(django_related_model, k, v)
                for k, v in wanted_attributes.items()
            }
            wanted_attributes['content_type_id'] = main_content_type.id
            wanted_attributes['object_id'] = django_object.id
            existing = django_related_model.objects.filter(**wanted_attributes)
            if existing.exists():
                old_objects_to_preserve += existing
            else:
                new_objects.append(
                    django_related_model.objects.create(**wanted_attributes)
                )
        object_ids_to_preserve = set(c.id for c in new_objects)
        object_ids_to_preserve.update(c.id for c in old_objects_to_preserve)
        django_related_model.objects.filter(
            content_type_id=main_content_type.id,
            object_id=django_object.id
        ).exclude(pk__in=object_ids_to_preserve).delete()

    def make_contact_detail_dict(self, contact_detail_data):
        contact_type = contact_detail_data.get('type') or \
                       contact_detail_data.get('contact_type')
        return {
            'label': contact_detail_data.get('label') or '',
            'contact_type': contact_type,
            'value': contact_detail_data['value'],
            'note': contact_detail_data.get('note') or '',
            'start_date': contact_detail_data.get('valid_from') or '',
            'end_date': contact_detail_data.get('valid_until') or '',
        }

    def make_link_dict(self, link_data):
        return {
            'note': link_data.get('note') or '',
            'url': link_data['url'],
        }

    def make_source_dict(self, source_data):
        return {
            'url': source_data['url'],
            'note': source_data.get('note') or '',
        }

    def make_other_name_dict(self, other_name_data):
        return {
            'name': other_name_data.get('name') or '',
            'note': other_name_data.get('note') or '',
            'start_date': other_name_data.get('start_date') or '',
            'end_date': other_name_data.get('end_date') or '',
        }

    def make_identifier_dict(self, identifier_data):
        return {
            'identifier': identifier_data['identifier'],
            'scheme': identifier_data['scheme'],
        }
